/**
 * HelpScout Beacon implementation. Can ask for permission before initialization and showing the beacon.
 */
class HsBeacon {
	/**
	 * Creates a Beacon object
	 * @param beaconId
	 */
	construction(beaconId: string) {
		this.beaconId = beaconId;
	}

	/**
	 * Attach ask and show event to the window object.
	 */
	attachEvents() {
		window.addEventListener('hsAskBeaconPermission', () => {
			this.askPermission();
		});
		window.addEventListener('hsShowBeacon', () => {
			this.show();
		});
	}

	/**
	 * Ask for permission to show the beacon.
	 */
	askPermission() {
		if (this.isPermissionGranted()) {
			this.show();
		} else {
			this.showPermissionRequest();
		}
	}

	/**
	 * Show the beacon.
	 */
	show() {
		if (this.isPermissionGranted()) {
			this.showBeacon();
		} else {
			this.showPermissionRequest();
		}
	}

	isPermissionGranted() {
		return this.hasPermission();
	}

	hasPermission() {
		return window.localStorage.getItem('hsBeaconPermission') === 'true';
	}

	/**
	 * Show the permission request.
	 */
	showPermissionRequest() {
		const permissionRequest = document.createElement('div');
		permissionRequest.id = 'hs-beacon-permission-request';
		permissionRequest.innerHTML = `
			<div class="hs-beacon-permission-request-container">
				<div class="hs-beacon-permission-request-content">
					<div class="hs-beacon-permission-request-title">
						<span>HelpScout Beacon</span>
					</div>
					<div class="hs-beacon-permission-request-body">
						<span>
							We use cookies to improve your experience on our website. By continuing to use our website, you agree to our use of cookies.
						</span>
					</div>
					<div class="hs-beacon-permission-request-buttons">
						<button class="hs-beacon-permission-request-button hs-beacon-permission-request-button-allow">Allow</button>
						<button class="hs-beacon-permission-request-button hs-beacon-permission-request-button-deny">Deny</button>
					</div>
				</div>
			</div>
		`;
		document.body.appendChild(permissionRequest);

		const allowButton = permissionRequest.querySelector('.hs-beacon-permission-request-button-allow');
		allowButton.addEventListener('click', () => {
			this.setPermission(true);
			this.showBeacon();
			permissionRequest.remove();
		});

		const denyButton = permissionRequest.querySelector('.hs-beacon-permission-request-button-deny');
		denyButton.addEventListener('click', () => {
			this.setPermission(false);
			permissionRequest.remove();
		});
	}

	/**
	 * Set the permission in the local storage.
	 * @param value
	 */
	setPermission(value: boolean) {
		window.localStorage.setItem('hsBeaconPermission', value.toString());
	}

	/**
	 * Show the beacon.
	 */
	showBeacon() {
		const beacon = document.createElement('div');
		beacon.id = 'hs-beacon';
		beacon.innerHTML = `
			<div class="hs-beacon-container">
				<div class="hs-beacon-content">
					<div class="hs-beacon-title">
						<span>HelpScout Beacon</span>
					</div>
					<div class="hs-beacon-body">
						<span>
							We use cookies to improve your experience on our website. By continuing to use our website, you agree to our use of cookies.
						</span>
					</div>
					<div class="hs-beacon-buttons">
						<button class="hs-beacon-button hs-beacon-button-close">Close</button>
					</div>
				</div>
			</div>
		`;
		document.body.appendChild(beacon);

		const closeButton = beacon.querySelector('.hs-beacon-button-close');
		closeButton.addEventListener('click', () => {
			beacon.remove();
		});
	}

}
