<?php

declare( strict_types=1 );

use Psr\Container\ContainerInterface;
use Psr\Log\LoggerInterface;
use WPDesk\ShopMagic\Admin;
use WPDesk\ShopMagic\Api\Normalizer\FieldNormalizer\ActionFieldNormalizer;
use WPDesk\ShopMagic\Api\Normalizer\FieldNormalizer\CheckboxFieldNormalizer;
use WPDesk\ShopMagic\Api\Normalizer\FieldNormalizer\DateFieldNormalizer;
use WPDesk\ShopMagic\Api\Normalizer\FieldNormalizer\JsonSchemaFieldNormalizer;
use WPDesk\ShopMagic\Api\Normalizer\FieldNormalizer\ModuleFieldNormalizer;
use WPDesk\ShopMagic\Api\Normalizer\FieldNormalizer\MultipleSelectFieldNormalizer;
use WPDesk\ShopMagic\Api\Normalizer\FieldNormalizer\NoValueFieldNormalizer;
use WPDesk\ShopMagic\Api\Normalizer\FieldNormalizer\ParagraphFieldNormalizer;
use WPDesk\ShopMagic\Api\Normalizer\FieldNormalizer\ProductSelectFieldNormalizer;
use WPDesk\ShopMagic\Api\Normalizer\FieldNormalizer\SelectFieldNormalizer;
use WPDesk\ShopMagic\Api\Normalizer\FieldNormalizer\TimeFieldNormalizer;
use WPDesk\ShopMagic\Api\Normalizer\NormalizerCollection;
use WPDesk\ShopMagic\Components\Routing\Controller\ArgumentResolver;
use WPDesk\ShopMagic\Frontend\Interceptor\CurrentCustomer;
use WPDesk\ShopMagic\Guest\GuestBackgroundConverter;
use WPDesk\ShopMagic\HookEmitter;
use WPDesk\ShopMagic\Integration\ExternalPluginsAccess;
use WPDesk\ShopMagic\Tracker;
use WPDesk\ShopMagic\Workflow\ActionExecution\ExecutionCreator\ExecutionCreator;
use WPDesk\ShopMagic\Workflow\ActionExecution\ExecutionCreator\ExecutionCreatorContainer;
use WPDesk\ShopMagic\Workflow\Queue\ActionSchedulerQueue;
use WPDesk\ShopMagic\Workflow\Queue\Queue;

use function ShopMagicVendor\DI\create;
use function ShopMagicVendor\DI\factory;
use function ShopMagicVendor\DI\get;

return [
	'beacon'                => '6057086f-4b25-4e12-8735-fbc556d2dc01',
	\WPDesk\ShopMagic\Helper\PluginBag::class => autowire(),
	'hookable.plugins_loaded'                 => [
		autowire( \WPDesk\ShopMagic\Workflow\ActionExecution\QueueActionRunner::class ),
		get( GuestBackgroundConverter::class ),
		autowire( HookEmitter\CronHeartbeat::class ),
		autowire( \ShopMagicVendor\WPDesk\Dashboard\DashboardWidget::class ),
	],
	GuestBackgroundConverter::class           => autowire(),
	'hookable.init'                           => [
		autowire( \ShopMagicVendor\WPDesk\Notice\AjaxHandler::class )
			->constructor( static function ( ContainerInterface $c ) {
				$plugin_url = $c->get( \WPDesk\ShopMagic\Helper\PluginBag::class )->get_url();

				return "$plugin_url/vendor_prefixed/wpdesk/wp-notice/assets";
			} ),
		autowire( \WPDesk\ShopMagic\Workflow\Automation\AutomationPostType::class ),
		autowire( \WPDesk\ShopMagic\Marketing\Segmentation\CommunicationListPostType::class ),
		autowire( Tracker\TrackerNotices::class ),
		autowire( \WPDesk\ShopMagic\Marketing\Subscribers\Controller\CustomerSignUp::class ),
		autowire( \WPDesk\ShopMagic\Marketing\Subscribers\Controller\PreferencesUpdate::class ),
		autowire( \WPDesk\ShopMagic\Marketing\Subscribers\View\ListsOnCheckout::class )
			->method( 'set_renderer', get( 'renderer.marketing' ) ),
		get( CurrentCustomer::class ),
		autowire( \WPDesk\ShopMagic\Guest\GuestOrderIntegration::class ),
		autowire( \WPDesk\ShopMagic\Guest\GuestProductIntegration::class ),
		get( \WPDesk\ShopMagic\Marketing\Subscribers\View\AccountPreferences::class ),
//		autowire( MarketingLists\PreferencesRoute::class )
//			->method( 'set_view', get( MarketingLists\View\AccountPreferences::class ) ),
		autowire( Tracker\DeactivationTracker::class ),
		autowire( Tracker\UsageDataTracker::class ),
		autowire( HookEmitter\OutcomeCleaner::class )
			->constructorParameter( 'persister', get( \WPDesk\ShopMagic\Workflow\Outcome\OutcomeManager::class ) ),
		autowire( \WPDesk\ShopMagic\Frontend\Interceptor\PreSubmitData::class ),
		autowire( \WPDesk\ShopMagic\Beacon\Beacon::class )
			->constructor(
				get( 'beacon' ),
				create( \ShopMagicVendor\WPDesk\ShowDecision\PostTypeStrategy::class )
					->constructor( \WPDesk\ShopMagic\Workflow\Automation\AutomationPostType::TYPE ),
				get( \WPDesk\ShopMagic\Helper\PluginBag::class )
			),
		autowire( \WPDesk\ShopMagic\Marketing\MailTracking\RecordEmailSending::class ),
		autowire( \WPDesk\ShopMagic\Marketing\Subscribers\Shortcode\FrontendForm::class )
			->method( 'set_renderer', get( 'renderer.marketing' ) ),
		autowire( \WPDesk\ShopMagic\Marketing\Subscribers\Shortcode\FrontendListSubscription::class ),
		autowire( \WPDesk\ShopMagic\Marketing\Subscribers\ConfirmedSubscriptionSaver::class ),
	],
	CurrentCustomer::class => autowire(),

	'renderer.marketing' => static function () {
		return new \ShopMagicVendor\WPDesk\View\Renderer\SimplePhpRenderer(
			\WPDesk\ShopMagic\Helper\TemplateResolver::for_public( 'marketing-lists' )
		);
	},
	\WPDesk\ShopMagic\Workflow\Outcome\OutcomeManager::class => autowire()
		->constructorParameter( 'repository', get( \WPDesk\ShopMagic\Workflow\Outcome\OutcomeRepository::class ) )
		->constructorParameter( 'normalizer', get( \WPDesk\ShopMagic\Workflow\Outcome\OutcomeHydrator::class ) ),

	\WPDesk\ShopMagic\Workflow\Outcome\OutcomeRepository::class          => autowire()
		->constructorParameter( 'denormalizer', get( \WPDesk\ShopMagic\Workflow\Outcome\OutcomeHydrator::class ) ),

	\WPDesk\ShopMagic\Workflow\Outcome\Meta\OutcomeMetaManager::class => autowire()
		->constructorParameter('repository', get(\WPDesk\ShopMagic\Workflow\Outcome\Meta\OutcomeMetaRepository::class))
		->constructorParameter('normalizer', get(\WPDesk\ShopMagic\Workflow\Outcome\Meta\OutcomeMetaHydrator::class)),

	\WPDesk\ShopMagic\Workflow\Outcome\Meta\OutcomeMetaRepository::class => autowire()
		->constructorParameter( 'denormalizer',
			get( \WPDesk\ShopMagic\Workflow\Outcome\Meta\OutcomeMetaHydrator::class ) ),

	\WPDesk\ShopMagic\Guest\DAO\GuestObjectManager::class => autowire()
		->constructorParameter( 'repository', get( \WPDesk\ShopMagic\Guest\DAO\GuestObjectRepository::class ) )
		->constructorParameter( 'normalizer', get( \WPDesk\ShopMagic\Guest\DAO\GuestEntityHydrator::class ) ),

	\WPDesk\ShopMagic\Guest\DAO\GuestObjectRepository::class => autowire()
		->constructorParameter( 'denormalizer', get( \WPDesk\ShopMagic\Guest\DAO\GuestEntityHydrator::class ) ),

	\WPDesk\ShopMagic\Guest\DAO\GuestMetaManager::class => autowire()
		->constructorParameter( 'repository', get( \WPDesk\ShopMagic\Guest\DAO\GuestMetaRepository::class ) )
		->constructorParameter( 'normalizer', get( \WPDesk\ShopMagic\Guest\DAO\GuestMetaFactory::class ) ),

	\WPDesk\ShopMagic\Guest\DAO\GuestMetaRepository::class => autowire()
		->constructorParameter( 'denormalizer', get( \WPDesk\ShopMagic\Guest\DAO\GuestMetaFactory::class ) ),

	\WPDesk\ShopMagic\Marketing\MailTracking\TrackedEmailObjectManager::class => autowire()
		->constructorParameter( 'repository', get( \WPDesk\ShopMagic\Marketing\MailTracking\TrackedEmailRepository::class ) )
		->constructorParameter( 'normalizer',
			get( \WPDesk\ShopMagic\Marketing\MailTracking\TrackedEmailHydrator::class ) ),

	\WPDesk\ShopMagic\Marketing\MailTracking\TrackedEmailRepository::class => autowire()
		->constructorParameter( 'denormalizer',
			get( \WPDesk\ShopMagic\Marketing\MailTracking\TrackedEmailHydrator::class ) ),

	\WPDesk\ShopMagic\Marketing\MailTracking\TrackedClickObjectManager::class => autowire()
		->constructorParameter( 'repository', get( \WPDesk\ShopMagic\Marketing\MailTracking\TrackedClickRepository::class ) )
		->constructorParameter( 'normalizer',
			get( \WPDesk\ShopMagic\Marketing\MailTracking\TrackedEmailClickFactory::class ) ),

	\WPDesk\ShopMagic\Marketing\MailTracking\TrackedClickRepository::class => autowire()
		->constructorParameter( 'denormalizer',
			get( \WPDesk\ShopMagic\Marketing\MailTracking\TrackedEmailClickFactory::class ) ),

	\WPDesk\ShopMagic\Marketing\Subscribers\DAO\SubscriptionManager::class => autowire()
		->constructorParameter( 'repository', get( \WPDesk\ShopMagic\Marketing\Subscribers\DAO\SubscriberObjectRepository::class ) )
		->constructorParameter( 'normalizer',
			get( \WPDesk\ShopMagic\Marketing\Subscribers\DAO\SingleListSubscriberHydrator::class ) ),

	\WPDesk\ShopMagic\Marketing\Subscribers\DAO\SubscriberObjectRepository::class => autowire()
		->constructorParameter( 'denormalizer',
			get( \WPDesk\ShopMagic\Api\Normalizer\SubscriberHydrator::class ) ),

	\WPDesk\ShopMagic\Marketing\Segmentation\AudienceListObjectManager::class => autowire()
		->constructorParameter(
			'repository',
			get( \WPDesk\ShopMagic\Marketing\Segmentation\AudienceListRepository::class )
		),

	\WPDesk\ShopMagic\Marketing\Subscribers\View\AccountPreferences::class => autowire()
		->constructorParameter(
			'renderer',
			get( 'renderer.marketing' )
		),

	Admin\Admin::class                                                           => autowire()
		->method( 'set_manifest', static function ( ContainerInterface $c ) {
			$plugin_bag = $c->get(\WPDesk\ShopMagic\Helper\PluginBag::class);
			return Admin\Manifest::from_file(
				$plugin_bag->get_manifest_path(),
				$plugin_bag->get_admin_assets_url()
			);
		} ),
	ExecutionCreatorContainer::class => autowire()
		->method( 'add_execution_creator',
			get( \WPDesk\ShopMagic\Workflow\ActionExecution\ExecutionCreator\QueueExecutionCreator::class ) ),
	ExecutionCreator::class          => get( ExecutionCreatorContainer::class ),
	Queue::class                     => get( ActionSchedulerQueue::class ),
	ExternalPluginsAccess::class     => autowire(),

	wpdb::class => static function () {
		global $wpdb;

		return $wpdb;
	},

	LoggerInterface::class => factory(
		static function ( \ShopMagicVendor\DI\Container $c ) {
			$logger     = new \ShopMagicVendor\Monolog\Logger( 'shopmagic' );
			$handler    = new \ShopMagicVendor\Monolog\Handler\ErrorLogHandler(
				\ShopMagicVendor\Monolog\Handler\ErrorLogHandler::OPERATING_SYSTEM,
				\Psr\Log\LogLevel::CRITICAL
			);
			$request_id = uniqid( '', true );
			$handler->setFormatter(
				new \ShopMagicVendor\Monolog\Formatter\LineFormatter( "[%datetime%] %channel%.%level_name%: %message% %context% %extra%, requestId: $request_id" )
			);
			$logger->pushHandler( $handler );
			if ( $c->get( \WPDesk\ShopMagic\Helper\PluginBag::class )->debug_enabled() ) {
				$handler->setLevel( \Psr\Log\LogLevel::DEBUG );
			}

			return $logger;
		}
	),
	'logger'               => get( LoggerInterface::class ),

	\WPDesk\ShopMagic\Workflow\ActionExecution\ExecuteNow::class => autowire()
		->constructorParameter( 'permit_exceptions', true ),

	\WPDesk\ShopMagic\Workflow\Automation\AutomationInitializer::class        => autowire(),
	\ShopMagicVendor\WPDesk\Migrations\Migrator::class                        => static function ( ContainerInterface $c
	) {
	$plugin_bag = $c->get(\WPDesk\ShopMagic\Helper\PluginBag::class);
		return \ShopMagicVendor\WPDesk\Migrations\WpdbMigrator::from_directories( [$plugin_bag->get_migrations_path()],
			'shopmagic_db' );
	},
	\WPDesk\ShopMagic\Components\Routing\Controller\ArgumentResolver::class   => static function (
		\ShopMagicVendor\DI\Container $c
	) {
		return new \WPDesk\ShopMagic\Components\Routing\Controller\ArgumentResolver( [
			new ArgumentResolver\RequestValueResolver(),
			new ArgumentResolver\ContainerValueResolver( $c->get( ContainerInterface::class ) ),
			new ArgumentResolver\ParameterValueResolver(),
			new ArgumentResolver\RawRequestValueResolver(),
			new ArgumentResolver\DefaultValueResolver(),
		] );
	},
	\WPDesk\ShopMagic\Components\Routing\Controller\ControllerResolver::class => create( \WPDesk\ShopMagic\Components\Routing\Controller\ContainerControllerResolver::class )
		->constructor( get( ContainerInterface::class ) ),

	\WPDesk\ShopMagic\Components\Routing\WpRoutesRegistry::class                  => autowire()
		->constructorParameter( 'configurator', get( 'routesConfigurator.public' ) ),
	\WPDesk\ShopMagic\Components\Routing\RestRoutesRegistry::class                => autowire()
		->constructorParameter( 'configurator', get( 'routesConfigurator.api' ) )
		->method( 'setLogger', get( LoggerInterface::class ) ),
	'routesConfigurator.public'                                                   => create( \WPDesk\ShopMagic\Components\Routing\RoutesConfigurator::class ),
	'routesConfigurator.api'                                                      => static function () {
		$configurator = new \WPDesk\ShopMagic\Components\Routing\RoutesConfigurator();
		$configurator->prefix( 'shopmagic/v1' );
		$configurator->authorize( static function () {
			return current_user_can( 'manage_options' ) || current_user_can( 'edit_others_shop_orders' );
		} );

		return $configurator;
	},
	\WPDesk\ShopMagic\DataSharing\TestProvider::class                             => static function (
		ContainerInterface $c
	) {
		$test_provider = new \WPDesk\ShopMagic\DataSharing\TestProvider();
		$test_provider->add_provider( $c->get( \WPDesk\ShopMagic\DataSharing\CustomerTestProvider::class ) );

		if ( \WPDesk\ShopMagic\Helper\WordPressPluggableHelper::is_plugin_active( 'woocommerce/woocommerce.php' ) ) {
			$test_provider->add_provider( $c->get( \WPDesk\ShopMagic\DataSharing\OrderTestProvider::class ) );
		}

		return $test_provider;
	},
	\WPDesk\ShopMagic\Api\Normalizer\FieldNormalizer\JsonSchemaNormalizer::class => static function () {
		return new \WPDesk\ShopMagic\Api\Normalizer\FieldNormalizer\JsonSchemaNormalizer(
			new NormalizerCollection(
				new ActionFieldNormalizer(),
				new ModuleFieldNormalizer(),
				new ParagraphFieldNormalizer(),
				new NoValueFieldNormalizer(),
				new CheckboxFieldNormalizer(),
				new ProductSelectFieldNormalizer(),
				new MultipleSelectFieldNormalizer(),
				new SelectFieldNormalizer(),
				new TimeFieldNormalizer(),
				new DateFieldNormalizer(),
				new JsonSchemaFieldNormalizer()
			)
		);
	},

	Admin\Settings\ModulesInfoContainer::class => static function () {
		return Admin\Settings\ModulesSettings::get_settings_persistence();
	},

	\WPDesk\ShopMagic\Modules\Mulitilingual\Customer\CustomerLanguagePersistence::class => autowire()
		->constructorParameter( 'customer', static function ( ContainerInterface $c ) {
			return $c->get( CurrentCustomer::class )->get_customer();
		} )
		->constructorParameter( 'language',
			get( \WPDesk\ShopMagic\Modules\Mulitilingual\Integration\BareHandler::class ) ),

	\WPDesk\ShopMagic\Modules\Mulitilingual\MultilingualModule::class => autowire(),
];
