<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Api\Controller;

use WPDesk\ShopMagic\Api\Normalizer\GuestNormalizer;
use WPDesk\ShopMagic\Customer\CustomerRepository;
use WPDesk\ShopMagic\Guest\DAO\GuestObjectRepository;

class CustomerController {

	/** @var CustomerRepository */
	private $repository;

	public function __construct( CustomerRepository $repository ) {
		$this->repository = $repository;
	}

	public function index(): \WP_REST_Response {
		return new \WP_REST_Response( $this->repository->find_all() );
	}

	public function guests(
		\WP_REST_Request $request,
		GuestObjectRepository $repository,
		GuestNormalizer $normalizer
	): \WP_REST_Response {
		return new \WP_REST_Response(
			$repository->find_by( ...$this->parse_params( $request ) )
			           ->map( \Closure::fromCallable( [ $normalizer, 'normalize' ] ) )
			           ->to_array()
		);
	}

	private function parse_params( \WP_REST_Request $request ): array {
		$page      = $request->get_param( 'page' );
		$page_size = $request->get_param( 'pageSize' );
		$filters   = $request->get_param( 'filters' );

		$criteria = [];
		//if ( isset( $filters['type'] ) && in_array( $filters['type'], [ 'opt_in', 'opt_out' ], true ) ) {
		//	$criteria['meta_key']   = 'type';
		//	$criteria['meta_value'] = sanitize_text_field( $filters['type'] );
		//}
		//
		//if ( isset( $filters['name'] ) ) {
		//	$criteria['s'] = sanitize_text_field( $filters['name'] );
		//}

		return [ $criteria, [], ( ( $page - 1 ) * $page_size ), $page_size ];
	}

	public function guests_count(
		\WP_REST_Request $request,
		GuestObjectRepository $repository
	): \WP_REST_Response {
		return new \WP_REST_Response(
			$repository->get_count( ...$this->parse_params( $request ) )
		);
	}
}
