<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Api\Controller;

use WPDesk\ShopMagic\Api\Normalizer\AutomationNormalizer;
use WPDesk\ShopMagic\Components\Collections\ArrayCollection;
use WPDesk\ShopMagic\Workflow\Automation\ValueObject\ActionValueObject;
use WPDesk\ShopMagic\Workflow\Automation\ValueObject\AutomationValueObject;
use WPDesk\ShopMagic\Workflow\Automation\ValueObject\EventValueObject;
use WPDesk\ShopMagic\Workflow\Automation\ValueObject\FilterValueObject;

class RecipesController {

	private const DEFAULT_LOCALE = 'en_US';

	/** @var string */
	private $dir;

	public function __construct() {
		$this->dir = __DIR__ . '/../../../config/recipes';
	}

	public function index( AutomationNormalizer $normalizer ): \WP_REST_Response {
		$locale       = $this->get_locale();
		$lang_path    = $this->dir . '/' . $locale;
		$recipe_files = (array) scandir( $lang_path );
		sort( $recipe_files, SORT_NATURAL );
		$recipes = ( new ArrayCollection( $recipe_files ) )
			->filter( static function ( string $filename ) use ( $lang_path ) {
				return is_file( $lang_path . '/' . $filename );
			} )
			->map( static function ( string $filename ) use ( $lang_path ): AutomationValueObject {
				$decoded       = json_decode( file_get_contents( $lang_path . '/' . $filename ), true );
				$as_automation = new AutomationValueObject();
				$as_automation->set_status( "draft" );
				$as_automation->set_name( $decoded['name'] );
				$as_automation->set_description( $decoded['description'] );
				$as_automation->set_event(
					new EventValueObject( $decoded['event']['slug'], $decoded['event']['data'] )
				);
				$as_automation->set_actions( array_map( static function ( $action ) {
					return new ActionValueObject( $action['_action'], $action );
				}, $decoded['actions'] ) );
				if ( ! empty( $decoded['filters'] ) ) {
					$as_automation->set_filters(
						array_map( static function ( $outer_group ) {
							return array_map( static function ( $inner_group ) {
								return new FilterValueObject(
									$inner_group['filter_slug'],
									$inner_group['data']
								);
							}, array_values( $outer_group ?? [] ) );
						}, array_values( $decoded['filters'] ) )
					);
				}
				$as_automation->set_from_recipe(true);

				return $as_automation;
			} )
			->map( \Closure::fromCallable( [ $normalizer, 'normalize' ] ) )
			->to_array();

		return new \WP_REST_Response( $recipes );
	}

	private function get_locale(): string {
		$locale = get_locale();
		if ( ! file_exists( $this->dir . '/' . $locale ) ) {
			$locale = self::DEFAULT_LOCALE;
		}

		return $locale;
	}

}
