<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Api\Normalizer\FieldNormalizer;

use ShopMagicVendor\WPDesk\Forms\Field;
use WPDesk\ShopMagic\Api\Normalizer\Normalizer;

class FieldNormalizer implements Normalizer {

	public function supports_normalization( object $field ): bool {
		return $field instanceof Field;
	}

	/**
	 * @param Field|object $field
	 *
	 * @return array
	 */
	public function normalize( object $field ): array {
		if ( ! $field instanceof Field ) {
			throw new \InvalidArgumentException( 'Wrong object for normalization' );
		}
		$rendered = [];
		foreach ( $field->get_attributes() as $key => $attribute ) {
			$rendered[ $key ] = $attribute;
		}
		$rendered['id']   = $field->get_name();
		$rendered['type'] = $field->get_type();
		if ( $field->get_type() === 'checkbox' && isset( $fields_data[ $field->get_name() ] ) ) {
			$rendered['value'] = $fields_data[ $field->get_name() ] === '1';
		} else {
			$rendered['value'] = $fields_data[ $field->get_name() ] ?? null;
		}
		if ( $field->get_default_value() ) {
			$rendered['defaultValue'] = $field->get_default_value();
		}
		if ( $field->has_description() ) {
			$rendered['description'] = $field->get_description();
		}
		if ( $field->has_description_tip() ) {
			$rendered['tip'] = $field->get_description_tip();
		}
		if ( $field->has_label() ) {
			$rendered['label'] = $field->get_label();
		}
		$options = $field->get_possible_values();
		if ( $options ) {
			$rendered['options'] = array_map(
				function ( $key, $value ) {
					return [
						'label' => $value,
						'value' => $key,
					];
				},
				array_keys( $options ),
				array_values( $options )
			);
		}
		if ( $field->has_data() ) {
			$data             = $field->get_data();
			$rendered['data'] = [];
			foreach ( $data as $data_name => $data_value ) {
				$rendered['data'][] = [
					'name'  => $data_name,
					'value' => $data_value,
				];
			}
		}

		if ( \json_encode( $rendered ) !== \false ) {
			return $rendered;
		}

		return [];
	}

}
