<?php
declare(strict_types=1);

namespace WPDesk\ShopMagic\Components\Database;

/**
 * Implementation for standard where/order/limit data parsing.
 */
trait RepositoryOrderingTrait {
	/**
	 * @param array<string, string> $order Order clauses in format [ field => asc|desc, field2 => asc|desc ]
	 */
	private function order_array_to_sql( array $order ): string {
		if ( empty( $order ) ) {
			return '';
		}

		$order_clauses = [];
		foreach ( $order as $key => $val ) {
			$order_clauses[] = sprintf( '%s %s', $key, $val );
		}

		return ' ORDER BY ' . $this->sanitize_sql_orderby( implode( ',', $order_clauses ) );
	}

	/**
	 *
	 * @see sanitize_sql_orderby()
	 */
	private function sanitize_sql_orderby( string $value ): string {
		if ( preg_match( '#^\s*(([a-z0-9_.]+|`[a-z0-9_.]+`)(\s+(ASC|DESC))?\s*(,\s*(?=[a-z0-9_`.])|$))+$#i', $value ) ) {
			return $value;
		}
		if ( preg_match( '#^\s*RAND\(\s*\)\s*$#i', $value ) ) {
			return $value;
		}
		if ( $this->logger ) {
			$this->logger->alert( sprintf( 'Invalid ORDER BY sanitization for value: %s', $value ) );
		}

		return '1=1';
	}

	private function limit_offset_to_sql( $limit, $offset ): string {
		return sprintf( ' LIMIT %s OFFSET %s', $limit, $offset );
	}

	private function where_array_to_sql( array $where ): string {
		if ( empty( $where ) ) {
			return ' 1 = 1 ';
		}

		$where_clauses = [];
		$where_values  = [];
		foreach ( $where as $key => $val ) {
			if ( \is_array( $val ) ) {
				$where_clauses[] = sprintf( '%s %s %%s', $val['field'], $val['condition'] );
				$where_values[]  = $val['value'];
			} else {
				$where_clauses[] = sprintf( '%s = %%s', $key );
				$where_values[]  = $val;
			}
		}

		return $this->get_wpdb()->prepare( ' ' . implode( ' AND ', $where_clauses ), $where_values );
	}
}
