<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Extensions\Builtin;

use WPDesk\ShopMagic\Extensions\AbstractExtension;
use WPDesk\ShopMagic\Integration\Mailchimp\AddToMailChimpListAction;
use WPDesk\ShopMagic\Marketing\Subscribers\DAO\SubscriberObjectRepository;
use WPDesk\ShopMagic\Workflow\Action\Builtin\OptinList\AddToListAction;
use WPDesk\ShopMagic\Workflow\Action\Builtin\OptinList\DeleteFromListAction;
use WPDesk\ShopMagic\Workflow\Action\Builtin\SendMail\SendMailAction;
use WPDesk\ShopMagic\Workflow\Action\Builtin\SendMail\SendPlainTextMailAction;
use WPDesk\ShopMagic\Workflow\Action\Builtin\SendMail\SendRawHTMLMailAction;
use WPDesk\ShopMagic\Workflow\Event\Builtin\Comment\CommentAdded;
use WPDesk\ShopMagic\Workflow\Event\Builtin\Customer\CustomerAccountCreated;
use WPDesk\ShopMagic\Workflow\Event\Builtin\Customer\CustomerOptedIn;
use WPDesk\ShopMagic\Workflow\Event\Builtin\Customer\CustomerOptedOut;
use WPDesk\ShopMagic\Workflow\Event\Builtin\Post\PostPublished;
use WPDesk\ShopMagic\Workflow\Event\Builtin\Post\PostUpdated;
use WPDesk\ShopMagic\Workflow\Filter\Builtin\Customer\CustomerIdFilter;
use WPDesk\ShopMagic\Workflow\Filter\Builtin\Customer\CustomerListFilter;
use WPDesk\ShopMagic\Workflow\Filter\Builtin\Customer\CustomerNotSubscribedToListFilter;
use WPDesk\ShopMagic\Workflow\Placeholder\Builtin\Customer;
use WPDesk\ShopMagic\Workflow\Placeholder\Builtin\Post;
use WPDesk\ShopMagic\Workflow\Placeholder\Builtin\Shop;
use WPDesk\ShopMagic\Workflow\Placeholder\Helper\PlaceholderUTMBuilder;

final class CoreExtension extends AbstractExtension {

	/** @var SubscriberObjectRepository */
	private $subscriber_repository;

	public function __construct( SubscriberObjectRepository $subscriber_repository ) {
		$this->subscriber_repository = $subscriber_repository;
	}

	public function get_actions(): array {
		return [
			'shopmagic_sendemail_action'          => SendMailAction::class,
			'shopmagic_plain_text_email_action'   => SendPlainTextMailAction::class,
			'shopmagic_raw_html_email_action'     => SendRawHTMLMailAction::class,
			'shopmagic_addtomailchimplist_action' => new AddToMailChimpListAction(),
			'shopmagic_add_to_list_action'        => AddToListAction::class,
			'shopmagic_delete_from_list_action'   => DeleteFromListAction::class,
		];
	}

	public function get_filters(): array {
		return [
			'customer_id'                 => new CustomerIdFilter(),
			'customer_communication_type' => CustomerListFilter::class,
			'customer_not_subscribed'     => CustomerNotSubscribedToListFilter::class,
		];
	}

	public function get_events(): array {
		return [
			'shopmagic_new_account_event'     => new CustomerAccountCreated(),
			'shopmagic_customer_optin_event'  => new CustomerOptedIn(),
			'shopmagic_customer_optout_event' => new CustomerOptedOut(),
			'shopmagic_post_published'        => new PostPublished(),
			'shopmagic_post_updated'          => new PostUpdated(),
			'shopmagic_comment_added'         => new CommentAdded(),
		];
	}

	public function get_placeholders(): array {
		return [
			new Customer\CustomerName(),
			new Customer\CustomerFirstName(),
			new Customer\CustomerLastName(),
			new Customer\CustomerEmail(),
			new Customer\CustomerPhone(),
			new Customer\CustomerUnsubscribeUrl(),
			new Customer\CustomerUsername(),

			new Post\PostTitle(),
			new Post\PostId(),
			new Post\PostLink(),
			new Post\PostContent(),

			new Shop\ShopTitle(),
			new Shop\ShopDescription(),
			new Shop\ShopUrl( new PlaceholderUTMBuilder() ),
		];
	}

}
