<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Guest\DAO;

use DateTimeInterface;
use WPDesk\ShopMagic\Components\Collections\ArrayCollection;
use WPDesk\ShopMagic\Components\Collections\Collection;

final class GuestEntity {

	/** @var int|null */
	private $id;
	/** @var string */
	private $email;
	/** @var string */
	private $tracking_key;
	/** @var DateTimeInterface */
	private $created;
	/** @var DateTimeInterface */
	private $updated;
	/** @var Collection<int, GuestMetaEntity> */
	private $meta;

	public function __construct() {
		$this->meta    = new ArrayCollection();
		$this->created = new \DateTimeImmutable();
		$this->updated = new \DateTimeImmutable();
	}

	/**
	 * @return string
	 */
	public function get_email(): string {
		return $this->email;
	}

	/**
	 * @param string $email
	 */
	public function set_email( string $email ): void {
		$this->email = $email;
	}

	/**
	 * @return string
	 */
	public function get_tracking_key(): string {
		return $this->tracking_key;
	}

	/**
	 * @param string $tracking_key
	 */
	public function set_tracking_key( string $tracking_key ): void {
		$this->tracking_key = $tracking_key;
	}

	/**
	 * @return DateTimeInterface
	 */
	public function get_created(): DateTimeInterface {
		return $this->created;
	}

	/**
	 * @param DateTimeInterface $created
	 */
	public function set_created( DateTimeInterface $created ): void {
		$this->created = $created;
	}

	/**
	 * @return DateTimeInterface
	 */
	public function get_updated(): DateTimeInterface {
		return $this->updated;
	}

	/**
	 * @param DateTimeInterface $updated
	 */
	public function set_updated( DateTimeInterface $updated ): void {
		$this->updated = $updated;
	}

	/**
	 * @return Collection<int, GuestMetaEntity>
	 */
	public function get_meta(): Collection {
		return $this->meta;
	}

	public function set_meta( Collection $meta ): void {
		$this->meta = $meta;
	}

	public function add_meta( GuestMetaEntity $meta ): void {
		$this->meta->set( $meta->get_meta_key(), $meta->get_meta_value() );
	}

	public function get_id(): ?int {
		return $this->id;
	}

	public function set_id( ?int $id ): void {
		$this->id = $id;
	}

}
