<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Guest\DAO;

use WPDesk\ShopMagic\Components\Collections\Collection;
use WPDesk\ShopMagic\Components\Database\Abstraction\DAO\ObjectDehydrator;
use WPDesk\ShopMagic\Components\Database\Abstraction\ObjectRepository;
use WPDesk\ShopMagic\Database\DatabaseTable;

/**
 * @extends ObjectRepository<GuestEntity>
 */
class GuestObjectRepository extends \WPDesk\ShopMagic\Components\Database\Abstraction\ObjectRepository {

	/** @var GuestMetaRepository */
	private $meta_repository;

	public function __construct(
		GuestMetaRepository $meta_repository,
		ObjectDehydrator $denormalizer,
		?\wpdb $wpdb = null
	) {
		$this->meta_repository = $meta_repository;
		parent::__construct( $denormalizer, $wpdb );
	}

	/**
	 * @param array      $criteria
	 * @param array|null $order
	 *
	 * @return GuestEntity
	 */
	public function find_one_by( array $criteria, ?array $order = null ): object {
		/** @var GuestEntity $guest */
		$guest = parent::find_one_by( $criteria, $order );
		$guest->set_meta( $this->meta_repository->find_by( [ 'guest_id' => $guest->get_id() ] ) );

		return $guest;
	}

	public function find_by( array $criteria, array $order = [], int $offset = 0, ?int $limit = null ): Collection {
		$guest_collection = parent::find_by( $criteria, $order, $offset, $limit );

		return $guest_collection->map( function ( GuestEntity $guest ) {
			$guest->set_meta(
				$this->meta_repository->find_by(
					[ 'guest_id' => $guest->get_id() ]
				)
			);

			return $guest;
		} );
	}

	public function find( $id ): object {
		/** @var GuestEntity $guest */
		$guest = parent::find( $id );
		$guest->set_meta( $this->meta_repository->find_by( [ 'guest_id' => $guest->get_id() ] ) );

		return $guest;
	}

	protected function get_name(): string {
		return DatabaseTable::guest();
	}
}
