<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Guest;

use WPDesk\ShopMagic\Guest\DAO\GuestEntity;
use WPDesk\ShopMagic\Guest\DAO\GuestMetaEntity;

final class Guest {

	/** @var GuestEntity */
	private $guest_entity;

	public function __construct( GuestEntity $guest_entity ) {
		$this->guest_entity = $guest_entity;
	}

	public function get_id(): string {
		return 'g_' . $this->guest_entity->get_id();
	}

	public function get_email(): string {
		return $this->guest_entity->get_email();
	}

	public function get_tracking_key(): string {
		return $this->guest_entity->get_tracking_key();
	}

	public function get_created(): \DateTimeInterface {
		return $this->guest_entity->get_created();
	}

	public function get_updated(): \DateTimeInterface {
		return $this->guest_entity->get_updated();
	}

	public function is_saved(): bool {
		return ! is_null( $this->guest_entity->get_id() );
	}

	public function get_meta_value( string $name ): string {
		return $this->guest_entity->get_meta()->get( $name ) ?? '';
	}

	public function set_meta_value( string $name, $value ) {
		$this->guest_entity->get_meta()->set( $name, $value );
	}

	public function has_meta_value( string $name ): bool {
		return $this->guest_entity->get_meta()->has( $name );
	}

	/** @return iterable<GuestMetaEntity> */
	public function get_all_metadata(): iterable {
		return $this->guest_entity->get_meta();
	}

	/** @internal */
	public function get_entity(): GuestEntity {
		return $this->guest_entity;
	}

}
