<?php
declare( strict_types=1 );

namespace WPDesk\ShopMagic\Guest;

use WPDesk\ShopMagic\Customer\Customer;
use WPDesk\ShopMagic\Guest\DAO\GuestEntity;

final class GuestCustomer implements Customer {

	/** @var Guest */
	private $guest;

	public function __construct( Guest $guest ) {
		$this->guest = $guest;
	}

	public static function from_entity( GuestEntity $entity ): Customer {
		$guest = new Guest($entity, $entity->get_meta());
		return new self($guest);
	}

	public function is_guest(): bool {
		return true;
	}

	public function get_id(): string {
		return $this->guest->get_id();
	}

	public function get_username(): string {
		return $this->guest->get_meta_value( 'username' );
	}

	public function get_first_name(): string {
		return $this->guest->get_meta_value( 'first_name' );
	}

	public function get_last_name(): string {
		return $this->guest->get_meta_value( 'last_name' );
	}

	public function get_full_name(): string {
		return $this->guest->get_meta_value( 'first_name' ) . ' ' . $this->guest->get_meta_value( 'last_name' );
	}

	public function get_email(): string {
		return $this->guest->get_email();
	}

	public function get_phone(): string {
		return $this->guest->get_meta_value( 'billing_phone' );
	}

	public function get_language(): string {
		return $this->guest->get_meta_value( Customer::USER_LANGUAGE_META );
	}
}
