<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Guest;

use WPDesk\ShopMagic\Components\Collections\ArrayCollection;
use WPDesk\ShopMagic\Guest\DAO\GuestEntity;
use WPDesk\ShopMagic\Guest\DAO\GuestEntityHydrator;
use WPDesk\ShopMagic\Guest\DAO\GuestMetaEntity;

final class GuestFactory {

	public function from_email( string $email ): Guest {
		$guest = new GuestEntity();
		$guest->set_email( $email );
		$guest->set_tracking_key( GuestEntityHydrator::generate_tracking_key() );

		return new Guest(
			$guest
		);
	}

	public function from_order( \WC_Order $order, ?Guest $current_guest = null ): Guest {
		$metadata                      = [];
		$metadata['first_name']        = $order->get_billing_first_name();
		$metadata['last_name']         = $order->get_billing_last_name();
		$metadata['billing_company']   = $order->get_billing_company();
		$metadata['billing_phone']     = $order->get_billing_phone();
		$metadata['billing_address_1'] = $order->get_billing_address_1();
		$metadata['billing_address_2'] = $order->get_billing_address_2();
		$metadata['billing_country']   = $order->get_billing_country();
		$metadata['billing_city']      = $order->get_billing_city();
		$metadata['billing_state']     = $order->get_billing_state();
		$metadata['billing_postcode']  = $order->get_billing_postcode();

		$metadata['shipping_company']   = $order->get_shipping_company();
		$metadata['shipping_address_1'] = $order->get_shipping_address_1();
		$metadata['shipping_address_2'] = $order->get_shipping_address_2();
		$metadata['shipping_country']   = $order->get_shipping_country();
		$metadata['shipping_city']      = $order->get_shipping_city();
		$metadata['shipping_state']     = $order->get_shipping_state();
		$metadata['shipping_postcode']  = $order->get_shipping_postcode();

		$meta = [];
		foreach ( $metadata as $key => $value ) {
			$m = new GuestMetaEntity();
			$m->set_meta_key( $key );
			$m->set_meta_value( $value );
			$meta[] = $m;
		}

		$guest = $current_guest->get_entity() ?? new GuestEntity();
		$guest->set_updated( $order->get_date_created() );
		if ( ! $current_guest ) {
			$guest->set_email( $order->get_billing_email() );
			$guest->set_created( $order->get_date_created() );
			$guest->set_tracking_key( GuestEntityHydrator::generate_tracking_key() );
		}
		$guest->set_meta( new ArrayCollection( $meta ) );

		return new Guest(
			$guest
		);
	}

}
