<?php
declare( strict_types=1 );

namespace WPDesk\ShopMagic\Guest;

use WPDesk\ShopMagic\Exception\CannotProvideItemException;

/**
 * Catches guests from newly created orders and puts them into our guest repository.
 */
final class GuestOrderIntegration {
	use GuestInOrderContextTrait;

	public const PRIORITY_BEFORE_DEFAULT = - 100;

	/** @var GuestRepository */
	private $repository;

	/** @var GuestFactory */
	private $factory;

	public function __construct( GuestRepository $repository, GuestFactory $factory ) {
		$this->repository = $repository;
		$this->factory    = $factory;
	}

	public function hooks(): void {
		add_action(
			'woocommerce_new_order',
			function ( int $order_id ): void {
				$this->catch_guest( $order_id );
			},
			self::PRIORITY_BEFORE_DEFAULT,
			2
		);
		add_action(
			'woocommerce_api_create_order',
			function ( int $order_id ): void {
				$this->catch_guest( $order_id );
			},
			self::PRIORITY_BEFORE_DEFAULT,
			2
		);
	}

	private function catch_guest( int $order_id ): void {
		$order = wc_get_order( $order_id );

		if ( ! $order instanceof \WC_Order ) {
			return;
		}

		if ( $this->order_has_guest( $order ) ) {
			try {
				$guest = $this->repository->get_by_email( $order->get_billing_email() );
			} catch ( CannotProvideItemException $e ) {
				$guest = null;
			}

			$updated_guest = $this->factory->from_order( $order, $guest );
			$this->repository->save( $updated_guest );
			$this->touch_order( $order->get_id(), $guest->get_entity()->get_id() );
		}
	}
}
