<?php
declare( strict_types=1 );

namespace WPDesk\ShopMagic\Guest;

use WPDesk\ShopMagic\Exception\CannotProvideItemException;

final class GuestProductIntegration {
	public const PRIORITY_BEFORE_DEFAULT = - 100;

	/** @var GuestRepository */
	private $repository;

	/** @var GuestFactory */
	private $factory;

	public function __construct( GuestRepository $repository, GuestFactory $factory ) {
		$this->repository = $repository;
		$this->factory    = $factory;
	}

	public function hooks(): void {
		add_action(
			'comment_post',
			function ( int $comment_ID ): void {
				$this->catch_guest( $comment_ID );
			},
			self::PRIORITY_BEFORE_DEFAULT
		);
	}

	private function catch_guest( int $comment_ID ): void {
		$comment = get_comment( $comment_ID );
		if ( empty( $comment->user_id ) && ! empty( $comment->comment_author_email ) ) {
			try {
				$guest = $this->repository->get_by_email( $comment->comment_author_email );
			} catch ( CannotProvideItemException $e ) {
				$guest = $this->factory->from_email( $comment->comment_author_email );
			}

			$this->repository->save( $guest );
		}
	}
}
