<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Marketing\MailTracking\Controller;

use ShopMagicVendor\Ramsey\Uuid\Uuid;
use WPDesk\ShopMagic\Exception\CannotProvideItemException;
use WPDesk\ShopMagic\Marketing\MailTracking\TrackedEmail;
use WPDesk\ShopMagic\Marketing\MailTracking\TrackedEmailHydrator;
use WPDesk\ShopMagic\Marketing\MailTracking\TrackedEmailObjectManager;
use WPDesk\ShopMagic\Marketing\MailTracking\TrackedEmailRepository;

class MailTrackingController {

	/** @var TrackedEmailRepository */
	private $repository;

	/** @var TrackedEmailHydrator */
	private $factory;

	/** @var TrackedEmailObjectManager */
	private $manager;

	public function __construct(
		TrackedEmailObjectManager $manager,
		TrackedEmailRepository $repository,
		TrackedEmailHydrator $factory
	) {
		$this->manager    = $manager;
		$this->repository = $repository;
		$this->factory    = $factory;
	}

	/**
	 * @param string $l Query parameter for original URI
	 * @param string $c Query parameter for message ID
	 *
	 * @return \WP_HTTP_Response
	 */
	public function click( string $l, string $c ): \WP_HTTP_Response {
		$uuid = Uuid::fromString( $c );
		/** @var TrackedEmail $tracked_email */
		try {
			$tracked_email = $this->repository->find_one_by( [ 'message_id' => $uuid->toString() ] );
		} catch ( CannotProvideItemException $e ) {
			return new \WP_HTTP_Response(
				null,
				\WP_Http::FOUND,
				[
					'Location' => esc_url_raw( $l ),
				]
			);
		}

		if ( ! $tracked_email->is_opened() ) {
			$tracked_email->mark_opened();
		}

		$tracked_email->append_click(
			$this->factory->new_click( $tracked_email, $l )
		);

		$this->manager->save( $tracked_email );

		return new \WP_HTTP_Response(
			null,
			\WP_Http::FOUND,
			[
				'Location' => esc_url_raw( $l ),
			]
		);
	}

	public function open( string $c ): \WP_HTTP_Response {
		$pixel = sprintf(
			'%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c',
			71, 73, 70, 56, 57, 97, 1, 0, 1, 0, 128, 255, 0, 192, 192, 192, 0, 0, 0, 33, 249, 4, 1, 0, 0, 0, 0, 44, 0, 0, 0, 0, 1, 0, 1, 0, 0, 2, 2, 68, 1, 0, 59
		);

		$uuid = Uuid::fromString( $c );
		/** @var TrackedEmail $tracked_email */
		$tracked_email = $this->repository->find_one_by( [ 'message_id' => $uuid->toString() ] );
		if ( ! $tracked_email->is_opened() ) {
			$tracked_email->mark_opened();
		}

		$this->manager->save( $tracked_email );

		return new \WP_HTTP_Response(
			$pixel,
			\WP_Http::OK,
			[
				'Content-Type'              => 'image/gif',
				'Content-Length'            => 42,
				'Pragma'                    => 'public',
				'Expires'                   => '0',
				'Cache-Control'             => 'must-revalidate, post-check=0, pre-check=0, private',
				'Content-Disposition'       => 'attachment; filename="blank.gif"',
				'Content-Transfer-Encoding' => 'binary',
			]
		);
	}

}
