<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Marketing\MailTracking;

use ShopMagicVendor\WPDesk\PluginBuilder\Plugin\Hookable;
use WPDesk\ShopMagic\Workflow\Action\Builtin\SendMail\EmailSending;

class RecordEmailSending implements Hookable {

	/** @var TrackedEmailHydrator */
	private $factory;

	/** @var TrackedEmailObjectManager */
	private $manager;

	public function __construct(
		TrackedEmailHydrator $factory,
		TrackedEmailObjectManager $manager
	) {
		$this->factory = $factory;
		$this->manager = $manager;
	}

	public function hooks() {
		add_filter(
			'shopmagic/core/action/send_mail/sending',
			function ( EmailSending $event ): EmailSending {
				return $this->record_email_sending( $event );
			}
		);
	}

	private function record_email_sending( EmailSending $event ): EmailSending {
		$tracked_email = $this->factory->fresh(
			$event->get_email(),
			$event->get_automation(),
			$event->get_customer()
		);

		$this->manager->save( $tracked_email );

		$tracking_injection = new TrackingInjection( $tracked_email );
		$email              = $event->get_email();
		if ( $email->is_html() ) {
			$email = $tracking_injection->inject_tracking_pixel( $event->get_email() );
		}
		$email = $tracking_injection->inject_link_tracker( $email );

		return $event->with_email( $email );
	}

}
