<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Marketing\MailTracking;

use WPDesk\ShopMagic\Components\Mailer\Email;

class TrackingInjection {

	/** @var TrackedEmail */
	private $tracked_email;

	public function __construct( TrackedEmail $tracked_email ) {
		$this->tracked_email = $tracked_email;
	}

	public function inject_tracking_pixel( Email $email ): Email {
		// Append the tracking url
		$tracking_pixel = '<img border=0 width=1 alt="" height=1 src="' . add_query_arg(
				[ 'c' => $this->tracked_email->get_message_id() ],
				get_site_url( null, 'track/sm-open' )
			) . '" />';

		$linebreak = uniqid();
		$email     = $email->message( str_replace( "\n", $linebreak, $email->message ) );

		if ( preg_match( '/^(.*<body[^>]*>)(.*)$/', $email->message, $matches ) ) {
			$email = $email->message( $matches[1] . $tracking_pixel . $matches[2] );
		} else {
			$email = $email->message( $email->message . $tracking_pixel );
		}
		$email = $email->message( str_replace( $linebreak, "\n", $email->message ) );

		return $email;
	}

	public function inject_link_tracker( Email $email ): Email {
		return $email->message( preg_replace_callback(
			'/(<a[^>]*href=["])([^"]*)/',
			function ( $matches ) {
				if ( empty( $matches[2] ) ) {
					$url = get_site_url();
				} else {
					$url = str_replace( '&amp;', '&', $matches[2] );
				}

				$rawurlencode = rawurlencode( $url );

				return $matches[1] . add_query_arg(
						[
							'l' => $rawurlencode,
							'c' => $this->tracked_email->get_message_id(),
						],
						get_home_url( null, 'track/sm-click' )
					);
			},
			$email->message
		) );
	}
}
