<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Marketing\Segmentation;

class AudienceList {
	public const TYPE_OPTOUT = 'opt_out';
	public const TYPE_OPTIN  = 'opt_in';

	/** @var int|null */
	private $id;

	/** @var string */
	private $type = 'opt_in';

	/** @var string */
	private $status = 'draft';

	/** @var string */
	private $name;

	/** @var NewsletterForm */
	private $newsletter_form;

	/** @var bool */
	private $checkout_available = true;

	/** @var string */
	private $checkout_label;

	/** @var string */
	private $checkout_description;

	/** @var string */
	private $language;

	public function __construct() {
		$this->newsletter_form = new NewsletterForm();
	}

	/**
	 * Backward compatibility for entity class.
	 * Introduced mainly because of template file, which differs with selected methods.
	 *
	 * @todo Remove in 4.0
	 *
	 * @param $name
	 * @param $arguments
	 *
	 * @return string
	 */
	public function __call( $name, $arguments ) {
		if ( $name === 'get_checkbox_label' ) {
			return $this->get_checkout_label();
		}

		if ( $name === 'get_checkbox_description' ) {
			return $this->get_checkout_description();
		}

		throw new \BadMethodCallException();
	}

	public function get_checkout_label(): string {
		return $this->checkout_label;
	}

	public function set_checkout_label( string $checkout_label ): void {
		$this->checkout_label = $checkout_label;
	}

	public function get_checkout_description(): string {
		return $this->checkout_description;
	}

	public function set_checkout_description( string $checkout_description ): void {
		$this->checkout_description = $checkout_description;
	}

	public function get_id(): ?int {
		return $this->id;
	}

	public function set_id( ?int $id ): void {
		$this->id = $id;
	}

	public function get_status(): string {
		return $this->status;
	}

	public function set_status( string $status ): void {
		$this->status = $status;
	}

	public function get_name(): string {
		return $this->name;
	}

	public function set_name( string $name ): void {
		$this->name = $name;
	}

	public function get_newsletter_form(): NewsletterForm {
		return $this->newsletter_form;
	}

	public function set_newsletter_form( NewsletterForm $newsletter_form ): void {
		$this->newsletter_form = $newsletter_form;
	}

	public function is_checkout_available(): bool {
		return $this->checkout_available;
	}

	public function set_checkout_available( bool $checkout_available ): void {
		$this->checkout_available = $checkout_available;
	}

	public function get_type(): string {
		return $this->type;
	}

	public function set_type( string $type ): void {
		if ( ! in_array( $type, [ self::TYPE_OPTIN, self::TYPE_OPTOUT ], true ) ) {
			throw new \InvalidArgumentException(
				sprintf(
					'List must be either `%s` or `%s`. Type `%s` is not allowed',
					self::TYPE_OPTIN,
					self::TYPE_OPTOUT,
					$type
				)
			);
		}
		$this->type = $type;
	}

	public function get_language(): string {
		return $this->language ?? get_bloginfo( 'language' );
	}

	public function set_language( string $language ): void {
		$this->language = $language;
	}

}
