<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Marketing\Segmentation;

use WPDesk\ShopMagic\Components\Database\Abstraction\EntityNotFound;

class AudienceListFactory {

	public function create( int $id ): AudienceList {
		$post = get_post( $id );
		if ( $post === null ) {
			throw new EntityNotFound( sprintf( 'Could not find marketing list %d', $id ) );
		}
		$persistence = new CommunicationListPersistence( $id );

		$list = new AudienceList();
		$list->set_id( $id );
		$list->set_name( $post->post_title );
		$list->set_status( $post->post_status );
		$list->set_type( $persistence->get( 'type' ) );
		$list->set_checkout_available( $persistence->get( 'checkout_available' ) === '1' );
		$list->set_checkout_label( $persistence->get( 'checkout_label' ) );
		$list->set_checkout_description( $persistence->get( 'checkout_description' ) );

		if ( $persistence->has( '_form_shortcode' ) ) {
			$form     = new NewsletterForm();
			$raw_form = $persistence->get( '_form_shortcode' );
			$form->set_agreement( $raw_form['agreement'] ?? '' );
			$form->set_show_name( $raw_form['name'] === '1' );
			$form->set_show_labels( $raw_form['labels'] === '1' );
			$form->set_double_opt_in( $raw_form['double_optin'] === '1' );
			$list->set_newsletter_form( $form );
		}

		if ( $persistence->has( 'lang' ) ) {
			$list->set_language( $persistence->get( 'lang' ) );
		}

		return $list;
	}
}
