<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Marketing\Segmentation;

use WPDesk\ShopMagic\Components\Collections\ArrayCollection;
use WPDesk\ShopMagic\Components\Collections\Collection;
use WPDesk\ShopMagic\Components\Database\Abstraction\DAO\ObjectRepository;

/**
 * @implements ObjectRepository<AudienceList>
 */
class AudienceListRepository implements ObjectRepository {

	/** @var AudienceListFactory */
	private $factory;

	public function __construct( AudienceListFactory $factory ) {
		$this->factory = $factory;
	}

	public function find( $id ): object {
		return $this->factory->create( $id );
	}

	public function find_all(): Collection {
		return $this->find_by( [] );
	}

	public function find_by( array $criteria, array $order = [], int $offset = 0, int $limit = null ): Collection {
		$args  = array_merge(
			[
				'post_status' => 'any',
			],
			$criteria,
			[
				'post_type'      => CommunicationListPostType::TYPE,
				'posts_per_page' => $limit ?? - 1,
				'paged'          => $offset,
			]
		);
		$query = new \WP_Query( $args );

		$posts = [];
		foreach ( $query->get_posts() as $post ) {
			$posts[] = $this->factory->create( $post->ID );
		}

		return new ArrayCollection( $posts );
	}

	public function find_checkout_viewable_items(): Collection {
		return $this->find_by( [
			'post_status' => 'publish',
			'meta_query'  => [
				[
					'key'   => 'checkout_available',
					'value' => '1',
				],
				[
					'key'   => 'type',
					'value' => AudienceList::TYPE_OPTIN,
				],
			],
		] );
	}

	public function find_soft_optin(): Collection {
		return $this->find_by( [
			'post_status' => 'publish',
			'meta_query'  => [
				[
					'key'   => 'type',
					'value' => AudienceList::TYPE_OPTOUT,
				],
			],
		] );
	}

	public function find_one_by( array $criteria, ?array $order = null ): object {
		$args  = array_merge(
			$criteria,
			[
				'post_type'      => CommunicationListPostType::TYPE,
				'posts_per_page' => 1,
			]
		);
		$query = new \WP_Query( $args );

		$posts = $query->get_posts();

		return $this->factory->create( $posts[0]->ID );
	}

	public function count( array $criteria ): int {
		global $wpdb;

		$type = CommunicationListPostType::TYPE;
		if ( ! empty( $criteria ) ) {
			return absint( $wpdb->get_var( "
				SELECT COUNT( * )
				FROM {$wpdb->posts} p
				JOIN {$wpdb->postmeta} pm
					ON p.id = pm.post_id
				WHERE p.post_type = '${type}'
				AND p.post_status in ('publish', 'draft')
				AND pm.meta_key = '{$criteria['meta_key']}'
				AND pm.meta_value = '{$criteria['meta_value']}'
		" ) );
		} else {
			return absint( $wpdb->get_var( "
				SELECT COUNT( * )
				FROM {$wpdb->posts} p
				WHERE p.post_type = '${type}'
				AND p.post_status in ('publish', 'draft')
			" ) );
		}
	}
}
