<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Marketing\Subscribers\Controller;

use Psr\Log\LoggerAwareTrait;
use WPDesk\ShopMagic\Exception\CannotProvideItemException;
use WPDesk\ShopMagic\Marketing\Segmentation\AudienceList;
use WPDesk\ShopMagic\Marketing\Segmentation\AudienceListRepository;
use WPDesk\ShopMagic\Marketing\Subscribers\CustomerSubscriberService;
use WPDesk\ShopMagic\Marketing\Subscribers\DAO\SubscriberObjectRepository;


final class CustomerSignUp {
	use LoggerAwareTrait;

	/** @var AudienceListRepository */
	private $newsletter_repository;

	/** @var SubscriberObjectRepository */
	private $subscriber_repository;

	/** @var CustomerSubscriberService */
	private $subscriber_service;

	public function __construct(
		CustomerSubscriberService $subscriber_service,
		SubscriberObjectRepository $subscriber_repository,
		AudienceListRepository $newsletter_repository
	) {
		$this->subscriber_service    = $subscriber_service;
		$this->subscriber_repository = $subscriber_repository;
		$this->newsletter_repository = $newsletter_repository;
	}

	public function hooks(): void {
		add_action(
			'woocommerce_checkout_order_processed',
			function ( $_, $__, $order ) {
				$this->save_checkout_optins( $order );
			},
			20,
			3
		);
		add_action( 'user_register', [ $this, 'optin_to_softs' ] );
	}

	/**
	 * @param \WC_Order $order
	 * @param mixed[]   $posted_data
	 */
	public function save_checkout_optins( $order ): void {
		foreach ( $this->newsletter_repository->find_all() as $type ) {
			if ( isset( $_POST['shopmagic_optin'][ $type->get_id() ] ) &&
			     $_POST['shopmagic_optin'][ $type->get_id() ] === 'yes' &&
			     ! $this->subscriber_repository->is_subscribed_to_list( $order->get_billing_email(), $type->get_id() )
			) {
				$this->subscriber_service->subscribe( $order->get_billing_email(), $type->get_id() );
			}

			if ( $type->get_type() === AudienceList::TYPE_OPTOUT ) {
				try {
					$this->subscriber_repository->get_subscribed_to_list( $order->get_billing_email(),
						$type->get_id() );
				} catch ( CannotProvideItemException $e ) {
					$this->subscriber_service->subscribe( $order->get_billing_email(), $type->get_id() );
				}
			}
		}
	}

	/**
	 * @internal
	 */
	public function optin_to_softs( int $user_id ): void {
		$user = get_user_by( 'id', $user_id );
		if ( ! $user instanceof \WP_User ) {
			return;
		}

		foreach ( $this->newsletter_repository->find_soft_optin() as $opt_out ) {
			$this->subscriber_service->subscribe( $user->user_email, $opt_out->get_id() );
		}
	}

}
