<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Marketing\Subscribers;

use WPDesk\ShopMagic\Customer\Customer;

final class PreferencesRoute {

	private function should_append_communication_page_to_account(): bool {
		return apply_filters( 'shopmagic/core/communication_type/account_page_show', true );
	}

	public static function get_slug(): string {
		return sanitize_title_with_dashes(
			apply_filters(
				'shopmagic/core/communication_type/account_page_slug',
				'communication-preferences'
			)
		);
	}

	public function create_preferences_url( Customer $customer ): string {
		if ( ! $customer->is_guest() && $this->should_append_communication_page_to_account() ) {
			return wc_get_account_endpoint_url( $this->get_slug() );
		}

		$link = trailingslashit( home_url() ) . $this->get_slug();

		return $link . '?' . http_build_query(
			[
				'hash' => md5( $customer->get_email() . SECURE_AUTH_SALT ),
				'id'   => $customer->get_id(),
			]
		);
	}

	public static function validate_hash( string $email, string $hash ): bool {
		return md5( $email . SECURE_AUTH_SALT ) === $hash;
	}

}
