<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Marketing\Subscribers\View;

use ShopMagicVendor\WPDesk\View\Renderer\Renderer;
use WPDesk\ShopMagic\Marketing\Segmentation\AudienceListRepository;
use WPDesk\ShopMagic\Marketing\Subscribers\DAO\SubscriberObjectRepository;


/**
 * Communication type info for customer. Optins/optouts.
 */
final class ListsOnCheckout {

	/** @var SubscriberObjectRepository */
	private $subscribers_repository;

	/** @var Renderer */
	private $renderer;

	/** @var AudienceListRepository */
	private $repository;

	public function __construct(
		AudienceListRepository $repository,
		SubscriberObjectRepository $subscriber_repository
	) {
		$this->repository             = $repository;
		$this->subscribers_repository = $subscriber_repository;
	}

	public function set_renderer( Renderer $renderer ): void {
		$this->renderer = $renderer;
	}

	public function hooks(): void {
		add_action(
			'woocommerce_checkout_after_terms_and_conditions',
			function (): void {
				$this->render_after_terms_optins();
			}
		);
	}

	public function render_after_terms_optins(): void {
		$viewable_items = $this->repository->find_checkout_viewable_items();
		foreach (
			$viewable_items as $type
		) {
			$this->renderer->output_render(
				'checkout_optin',
				[
					'type'     => $type,
					'opted_in' => $this->subscribers_repository->is_subscribed_to_list( $this->get_email(),
						$type->get_id() ),
				]
			);
		}
	}

	private function get_email(): string {
		$session_data = WC()->session->get( 'customer' );
		if ( isset( $session_data['email'] ) ) {
			return sanitize_email( $session_data['email'] );
		}

		return '';
	}
}
