<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Modules;

use Psr\Container\ContainerInterface;

final class ModulesSet {

	/** @var array<class-string<Module>, Module> */
	private $modules = [];

	/** @var bool */
	private $initialized = false;

	/** @var ContainerInterface */
	private $container;

	public function __construct( ContainerInterface $container ) {
		$this->container = $container;
	}

	public function add_module( Module $module ): void {
		$name = $module->get_name();
		if ( $this->initialized ) {
			throw new \LogicException( sprintf( 'Unable to register extension "%s" as extensions have already been initialized.',
				$name ) );
		}
		$this->modules[ $name ] = $module;
	}

	public function init_modules(): void {
		foreach ( $this->modules as $module ) {
			$module->set_container( $this->container );
			if ( $module->is_enabled() ) {
				$module->initialize();
			}
		}

		$this->initialized = true;
	}

	public function get_modules(): array {
		return $this->modules;
	}

}
