<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Workflow\Action;

use Psr\Container\ContainerInterface;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerAwareTrait;
use ShopMagicVendor\WPDesk\Forms\Field\InputTextField;
use ShopMagicVendor\WPDesk\Forms\FieldProvider;
use ShopMagicVendor\WPDesk\Forms\FieldsDataReceiver;
use WPDesk\ShopMagic\DataSharing\DataReceiver;
use WPDesk\ShopMagic\DataSharing\Traits\DataReceiverAsProtectedField;
use WPDesk\ShopMagic\DataSharing\Traits\FieldsDataAsProtectedField;
use WPDesk\ShopMagic\DataSharing\Traits\StandardWooCommerceDataProviderAccessors;
use WPDesk\ShopMagic\Extensions\Elements\NamedElement;
use WPDesk\ShopMagic\Workflow\Placeholder\PlaceholderProcessor;

/**
 * Action is one of the major components in ShopMagic. When an event occurs the actions are executed.
 * Action can be serialized to json to save its data to queue.
 * Actions are retrieved using Prototype Pattern.
 */
abstract class Action implements LoggerAwareInterface, NamedElement, DataReceiver, FieldProvider, ActionLogic,
                                 FieldsDataReceiver, \JsonSerializable {
	use DataReceiverAsProtectedField;
	use FieldsDataAsProtectedField;
	use StandardWooCommerceDataProviderAccessors;
	use LoggerAwareTrait;

	/** @var PlaceholderProcessor */
	protected $placeholder_processor;

	public function get_description(): string {
		return '';
	}

	/**
	 * Processor is required to process placeholders that can be used in action fields.
	 */
	public function set_placeholder_processor( PlaceholderProcessor $processor ): void {
		$this->placeholder_processor = $processor;
	}

	/**
	 * Action has fields that can be shown in admin panel. Here are the values of these fields.
	 */
	public function get_fields_data(): ContainerInterface {
		return $this->fields_data;
	}

	public function __clone() {
		$this->fields_data = null;
	}

	public function get_fields(): array {
		return [
			( new InputTextField() )
				->set_name( '_action_title' )
				->set_label( esc_html__( 'Description', 'shopmagic-for-woocommerce' ) )
				->set_priority( 1 ),
		];
	}

	/**
	 * @deprecated Actions are not really serialized - any of them isn't actually used when deserializing.
	 * @return mixed[]
	 */
	public function jsonSerialize(): array {
		return [];
	}
}
