<?php
declare( strict_types=1 );

namespace WPDesk\ShopMagic\Workflow\Automation;

use WPDesk\ShopMagic\Workflow\Action\Action;
use WPDesk\ShopMagic\Workflow\Event\Event;
use WPDesk\ShopMagic\Workflow\Filter\FilterLogic;

final class Automation implements \JsonSerializable {

	/** @var int */
	private $id;

	/** @var Event */
	private $event;

	/** @var Action[] */
	private $actions;

	/** @var FilterLogic */
	private $filters;

	/** @param Action[]    $actions */
	public function __construct(
		int $automation_id,
		Event $event,
		FilterLogic $filters,
		array $actions
	) {
		$this->id      = $automation_id;
		$this->event   = $event;
		$this->filters = $filters;
		$this->actions = $actions;
	}

	public function get_name(): string {
		$post = get_post( $this->id );
		if ( $post !== null ) {
			return $post->post_title;
		}

		return '';
	}

	public function get_id(): int {
		return $this->id;
	}

	public function get_event(): Event {
		return $this->event;
	}

	/** @return Action[] */
	public function get_actions(): array {
		return $this->actions;
	}

	public function get_filters(): FilterLogic {
		return $this->filters;
	}

	public function has_action( int $index ): bool {
		return isset( $this->actions[ $index ] );
	}

	public function get_action( int $index ): Action {
		return $this->actions[ $index ];
	}

	public function get_status(): string {
		$post = get_post( $this->id );
		if ( $post !== null ) {
			return $post->post_status;
		}
		return '';
	}

	/** @return array{id: int} */
	public function jsonSerialize(): array {
		return [
			'id' => $this->id,
		];
	}

}
