<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Workflow\Automation;

use WPDesk\ShopMagic\Helper\PostMetaContainer;

/**
 * Responsible for save/load all automation data.
 */
final class AutomationPersistence extends PostMetaContainer implements PersistentAutomation {

	/** @var string */
	private const META_KEY_ACTIONS = '_actions';

	/** @var string */
	private const META_KEY_EVENT = '_event_data';

	/** @var string */
	public const FILTER_SLUG_KEY = 'filter_slug';

	/** @var string */
	public const FILTER_DATA_KEY = 'data';

	/** @var string */
	private const FILTERS_META_KEY = 'filters';

	public function get_automation_name(): string {
		return get_the_title( $this->post_id );
	}

	public function set_event_data( array $data, string $slug ): void {
		$this->set( self::META_KEY_EVENT, $data );
		$this->set( '_event', $slug );
	}

	public function set( string $id, $value ): bool {
		try {
			return parent::set( $id, $value );
		} catch ( \RuntimeException $e ) {
			throw new AutomationNotSaved( $e->getMessage() );
		}
	}

	public function set_actions_data( array $data ): void {
		foreach ( $data as $key => $action_data ) {
			$data_with_deprecation = apply_filters_deprecated(
				'shopmagic_settings_save',
				[ $action_data, $action_data ],
				'3.0.0',
				'shopmagic/core/settings/before_save'
			);
			$data[ $key ]          = apply_filters( 'shopmagic/core/settings/before_save', $data_with_deprecation );
		}

		$this->set( self::META_KEY_ACTIONS, $data );
	}

	public function set_filters_data( array $data ): void {
		$this->set( self::FILTERS_META_KEY, $data );
	}

	/**
	 * @return array|array<int, array<int, array<string, string|array<string, mixed>>>>
	 */
	public function get_filters_data(): array {
		return $this->fallback_to_array( $this->data[ self::FILTERS_META_KEY ] ?? [] );
	}

	/**
	 * @param mixed $mixed
	 *
	 * @return array
	 */
	private function fallback_to_array( $mixed ): array {
		return \is_array( $mixed ) ? $mixed : [];
	}

	public function get_event_slug(): string {
		return $this->data['_event'] ?? '';
	}

	/**
	 * @return array
	 */
	public function get_event_data(): array {
		return $this->fallback_to_array( $this->data[ self::META_KEY_EVENT ] ?? [] );
	}

	/**
	 * @return array
	 */
	public function get_actions_data(): array {
		return $this->fallback_to_array( $this->data[ self::META_KEY_ACTIONS ] ?? [] );
	}

}
