<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Workflow\Automation;

use WPDesk\ShopMagic\Components\Collections\ArrayCollection;
use WPDesk\ShopMagic\Components\Collections\Collection;
use WPDesk\ShopMagic\Components\Database\Abstraction\DAO\ObjectRepository;
use WPDesk\ShopMagic\Workflow\Automation\ValueObject\AutomationValueFactory;
use WPDesk\ShopMagic\Workflow\Automation\ValueObject\AutomationValueObject;

/**
 * @implements ObjectRepository<AutomationValueObject>
 */
class AutomationValueObjectRepository implements ObjectRepository {

	/** @var AutomationValueFactory */
	private $automation_factory;

	public function __construct( AutomationValueFactory $automation_factory ) {
		$this->automation_factory = $automation_factory;
	}

	public function find( $id ): object {
		return $this->automation_factory->create( $id );
	}

	public function find_all(): Collection {
		return $this->find_by(
			[
				'post_status' => 'any',
			]
		);
	}

	public function find_by( array $criteria, array $order = [], int $offset = 0, ?int $limit = null ): Collection {
		$args  = array_merge(
			$criteria,
			[
				'post_type'      => AutomationPostType::TYPE,
				'posts_per_page' => $limit ?? - 1,
				'offset'         => $offset,
			]
		);
		$query = new \WP_Query( $args );

		$posts = [];
		foreach ( $query->get_posts() as $post ) {
			$posts[] = $this->automation_factory->create( $post->ID );
		}

		return new ArrayCollection( $posts );
	}

	public function find_one_by( array $where, ?array $order = null ): object {
		$posts = get_posts();

		return $posts[0];
	}

	public function count( array $criteria = [] ): int {
		$args  = array_merge(
			$criteria,
			[
				'post_type'      => AutomationPostType::TYPE,
				'posts_per_page' => - 1,
			]
		);
		$query = new \WP_Query( $args );

		return count( $query->get_posts() );
	}

}
