<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Workflow\Automation\ValueObject;

class ActionValueObject {

	/** @var string */
	private $name;

	/** @var array */
	private $settings;

	public function __construct( string $name, array $settings ) {
		$this->name     = $name;
		$this->settings = $settings;
		$this->ensure_linebreak();
	}

	/**
	 * TinyMCE v4 sometimes uses '\r\n', etc. for line breaks, which is not recognised in TinyMCE v6.
	 * Until ShopMagic v4 we will mitigate this issue by manually changing text value.
	 *
	 * @todo remove in ShopMagic 4.0
	 */
	private function ensure_linebreak(): void {
		if ( $this->name === 'shopmagic_sendemail_action' ) {
			$this->settings['message_text'] = str_replace(
				[ '\r\n', '\n' ],
				'<br />',
				$this->settings['message_text'] ?? '' );
		}
	}

	public function get_name(): string {
		return $this->name;
	}

	public function get_settings(): array {
		return $this->settings;
	}

}
