<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Workflow\Automation\ValueObject;

use WPDesk\ShopMagic\Exception\AutomationNotFound;
use WPDesk\ShopMagic\Workflow\Automation\AutomationPersistence;
use WPDesk\ShopMagic\Workflow\Automation\PersistentAutomation;

class AutomationValueFactory {

	public function create( int $id ): AutomationValueObject {
		$post = get_post( $id );
		if ( $post === null ) {
			throw new AutomationNotFound( sprintf( 'Could not find automation %d', $id ) );
		}
		$persistence = new AutomationPersistence( $id );

		$automation = new AutomationValueObject();
		$automation->set_id( $id );
		$automation->set_name( $post->post_title );
		$automation->set_status( $post->post_status );
		$automation->set_event( $this->get_event( $persistence ) );
		$automation->set_actions( $this->get_actions( $persistence ) );
		$automation->set_filters( $this->get_filters( $persistence ) );
		if ( $persistence->has( 'lang' ) ) {
			$automation->set_language( $persistence->get( 'lang' ) );
		}

		if ( $persistence->has( 'shopmagic_source_recipe' ) ) {
			$automation->set_from_recipe( true );
		}

		return $automation;
	}

	private function get_event( PersistentAutomation $persistence ): EventValueObject {
		return new EventValueObject(
			$persistence->get_event_slug(),
			$persistence->get_event_data()
		);
	}

	/** @return \WPDesk\ShopMagic\Workflow\Action\Action[] */
	private function get_actions( PersistentAutomation $persistence ): array {
		return array_map(
			function ( ?array $action_data ): ActionValueObject {
				return new ActionValueObject(
					$action_data['_action'] ?? '',
					$action_data ?? []
				);
			},
			$persistence->get_actions_data() ?? []
		);
	}

	private function get_filters( PersistentAutomation $persistence ): array {
		$filters = [];
		foreach ( $persistence->get_filters_data() as $or_group_index => $and_group ) {
			$filters[ $or_group_index ] = [];
			foreach ( $and_group as $and_group_index => $data ) {
				if ( ! empty( $data[ AutomationPersistence::FILTER_DATA_KEY ] ) ) {
					$filter = new FilterValueObject(
						$data['filter_slug'] ?? '',
						$data['data']
					);

					$filters[ $or_group_index ][ $and_group_index ] = $filter;
				}
			}
		}

		return $filters;
	}
}
