<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Workflow\Automation\ValueObject;

class FilterValueObject {

	/** @var string */
	private $id;

	/** @var array<string, string|string[]> */
	private $settings;

	public function __construct( string $id, array $settings ) {
		$this->id       = $id;
		$this->settings = $this->normalize_settings( $settings );
	}

	/**
	 * To achieve compatibility between SM 2.x and 3.x all filter values
	 * MUST be presented as array, not map.
	 * Normalize those values during class instantiation.
	 *
	 * @param array $settings
	 *
	 * @return array
	 */
	private function normalize_settings( array $settings ): array {
		return array_map(
			static function ( $setting ) {
				if ( is_array( $setting ) ) {
					return array_values( $setting );
				}

				return $setting;
			},
			$settings
		);
	}

	public function get_id(): string {
		return $this->id;
	}

	public function get_settings(): array {
		return $this->settings;
	}

}
