<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Workflow\Event\Builtin;

use WPDesk\ShopMagic\Customer\Customer;
use WPDesk\ShopMagic\FormField\Field\SelectField;
use WPDesk\ShopMagic\Marketing\Segmentation\CommunicationListRepository;
use WPDesk\ShopMagic\Workflow\Event\ManualGlobalEvent;


abstract class OptCommonEvent extends UserCommonEvent implements ManualGlobalEvent {
	/** @var string */
	private const PARAM_COMMUNICATION_TYPE = 'communication_type';

	public static function trigger( array $args ): void {
		// TODO: refactor to use ManualGlobalEvent disptcher rather than late static binding.
		do_action( static::get_internal_action_name(), ...$args );
	}

	/**
	 * @throws \Exception
	 * TODO: refactor to use ManualGlobalEvent disptcher rather than late static binding
	 */
	protected static function get_internal_action_name(): string {
		throw new \Exception( 'Invalid get_internal_action_name call' );
	}

	public function get_fields(): array {
		return [
			( new SelectField() )
				->set_label( __( 'List', 'shopmagic-for-woocommerce' ) )
				->set_name( self::PARAM_COMMUNICATION_TYPE )
				->set_placeholder( __( 'Any list', 'shopmagic-for-woocommerce' ) )
				->set_options( CommunicationListRepository::get_lists_as_select_options() )
				->set_description_tip(
					__(
						'Choose the list to which the customer opted in, or from which he opted out.',
						'shopmagic-for-woocommerce'
					)
				),
		];
	}

	public function initialize(): void {
		add_action(
			$this->get_internal_action_name(),
			function ( \WPDesk\ShopMagic\Customer\Customer $customer, int $term_id ) {
				$this->process_event( $customer, $term_id );
			},
			10,
			2
		);
	}

	/**
	 * Save params and run actions.
	 */
	public function process_event( Customer $customer, int $term_id ): void {
		$this->resources->set( Customer::class, $customer );
		$this->customer = $customer;

		if ( ! $this->fields_data->has( self::PARAM_COMMUNICATION_TYPE ) ) {
			$this->logger->warning( sprintf( "Prevented event dispatch due to insufficient configuration. Missing '%s' parameter",
				self::PARAM_COMMUNICATION_TYPE ) );
		}

		$expected_type_id = $this->fields_data->get( self::PARAM_COMMUNICATION_TYPE );
		if ( empty( $expected_type_id ) || (int) $expected_type_id === $term_id ) {
			$this->trigger_automation();
		}
	}
}
