<?php
declare( strict_types=1 );

namespace WPDesk\ShopMagic\Workflow\Event\Builtin\Order;

use WC_Order;
use WPDesk\ShopMagic\Customer\Customer;
use WPDesk\ShopMagic\Workflow\Event\Builtin\OrderCommonEvent;
use WPDesk\ShopMagic\Workflow\Event\EventMutex;

final class OrderNew extends OrderCommonEvent {

	/** @var EventMutex */
	private $event_mutex;

	public function __construct( EventMutex $event_mutex ) {
		$this->event_mutex = $event_mutex;
	}

	public function get_name(): string {
		return __( 'New Order', 'shopmagic-for-woocommerce' );
	}

	public function get_description(): string {
		return __( 'Run automation when any new order is created. Use this event if the status of the processed order does not matter.', 'shopmagic-for-woocommerce' );
	}

	public function initialize(): void {
		add_action(
			'woocommerce_new_order',
			function ( $order_id, $order ) {
				$this->process_event( $order_id, $order );
			},
			self::PRIORITY_AFTER_DEFAULT,
			2
		);
		add_action(
			'woocommerce_api_create_order',
			function ( $order_id, $order ) {
				$this->process_event( $order_id, $order );
			},
			self::PRIORITY_AFTER_DEFAULT,
			2
		);
	}

	public function process_event( $order_id, $order ): void {
		if ( $this->event_mutex->check_uniqueness_once( spl_object_hash( $this ), [ 'order_id' => $order_id ] ) ) {
			$this->resources->set( WC_Order::class, $order );
			$this->resources->set(
				Customer::class,
				$this->customer_repository->find_by_email( $this->get_order()->get_billing_email() )
			);
			$this->trigger_automation();
		}
	}
}
